using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using DB = DropDownListLinquify.DB;

/*
Primary Objects Linquify - Business Class Generator
Copyright (C) 2010 Kory Becker
http://www.primaryobjects.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

namespace DropDownListLinquify.Types.GeneratedTypes.BaseTypes
{
    [Serializable]
    public class Color : BusinessBaseType<DB.Color, DB.peopleEntities1>
    {
        public class Where
        {
			public bool ColorIdSet;
			public bool PersonIdSet;
			public bool FloatValueSet;
			public bool NameSet;
			public bool ValueSet;

			private System.Nullable<int> _ColorId;
			private System.Nullable<global::System.Guid> _PersonId;
			private global::System.Nullable<double> _FloatValue;
			private string _Name;
			private System.Nullable<int> _Value;


			public System.Nullable<int> ColorId { get { return _ColorId; } set { _ColorId = value; ColorIdSet = true; } }
			public System.Nullable<global::System.Guid> PersonId { get { return _PersonId; } set { _PersonId = value; PersonIdSet = true; } }
			public global::System.Nullable<double> FloatValue { get { return _FloatValue; } set { _FloatValue = value; FloatValueSet = true; } }
			public string Name { get { return _Name; } set { _Name = value; NameSet = true; } }
			public System.Nullable<int> Value { get { return _Value; } set { _Value = value; ValueSet = true; } }


            public string MyWhere = "", MyOrderBy = "", MyTop = "";

            public Where()
            {
				ColorId = null;
				PersonId = null;
				FloatValue = null;
				Name = null;
				Value = null;

				ColorIdSet = false;
				PersonIdSet = false;
				FloatValueSet = false;
				NameSet = false;
				ValueSet = false;

            }
        };

		public int ColorId { get; set; }
		public System.Nullable<global::System.Guid> PersonId { get; set; }
		public global::System.Nullable<double> FloatValue { get; set; }
		public string Name { get; set; }
		public int Value { get; set; }

        public Where WhereClause { get; set; }

        #region Constructors

        public Color()
        {
            WhereClause = new Where();
        }

        public Color(System.Nullable<int> _ColorId, System.Nullable<global::System.Guid> _PersonId, global::System.Nullable<double> _FloatValue, string _Name, int _Value)
            : this()
        {
            if (_ColorId != null)
            {
                ColorId = (int)_ColorId;
            }

			PersonId = _PersonId;
			FloatValue = _FloatValue;
			Name = _Name;
			Value = _Value;

        }

        public Color(DB.Color colorLinq)
            : this()
        {
            Color color = new Color();
            Converter.ConvertToBusinessObject<Color>(color, colorLinq);

            Initialize(color);
        }

        #endregion

        private void Initialize(Color color)
        {
			ColorId = color.ColorId;
			PersonId = color.PersonId;
			FloatValue = color.FloatValue;
			Name = color.Name;
			Value = color.Value;

            OnInitializeComplete();
        }

        #region Linq Helpers

        public bool Save()
        {
			OnSave();
			
            if (Save<int, Color>(this, ColorId, false))
            {
				OnSaveComplete();			
				return true;
			}
			
			return false;
        }

		public bool Insert()
		{
			// Allows you to force an Insert via the Save function, rather than an Update. Aka, explicit insert.
			// Useful for cases where you want to assign a primary key before inserting, rather than auto-assigning.
			OnSave();
			
            if (Save<int, Color>(this, ColorId, true))
            {
				OnSaveComplete();			
				return true;
			}
			
			return false;
		}
		
        public bool Load(int _ColorId)
        {
            DB.Color colorLinq = Load<int>(_ColorId);
            
            if (colorLinq == null)
            {
				return false;
            }
            
            Color color = new Color();
            Converter.ConvertToBusinessObject<Color>(color, colorLinq);
            
            Initialize(color);
            
            return true;
        }

        public bool Delete()
        {
	    OnDelete();

            DB.Color colorLinq = Delete<int>(ColorId);

			if (colorLinq == null)
			{
				return false;
			}
			
            Color color = new Color();
            Converter.ConvertToBusinessObject<Color>(color, colorLinq);

            Initialize(color);
            
	    OnDeleteComplete();

            return true;
        }

        public List<Color> Find()
        {
            List<DB.Color> listLinq = Find(WhereClause);
            
            List<Color> list = new List<Color>();
            foreach (DB.Color color in listLinq)
            {
                list.Add(new Color(color));
            }

            return list;
        }

        public new List<Color> ToList()
        {
            List<DB.Color> listLinq = base.ToList();

            List<Color> list = new List<Color>();
            foreach (DB.Color color in listLinq)
            {
                list.Add(new Color(color));
            }

            return list;
        }

        #endregion

        #region Overridables

        /// <summary>
        /// Fired after the object is created or loaded.
        /// </summary>
        protected virtual void OnInitializeComplete()
        {
			WhereClause.ColorId = ColorId;
        }

        /// <summary>
        /// Fired before the object is inserted or updated.
        /// </summary>
        protected virtual void OnSave()
        {
        }

        /// <summary>
        /// Fired after the object is inserted or updated.
        /// </summary>
        protected virtual void OnSaveComplete()
        {
			WhereClause.ColorId = ColorId;
        }

        /// <summary>
        /// Fired before the object is deleted.
        /// </summary>
	protected virtual void OnDelete()
	{
	}

        /// <summary>
        /// Fired after the object is deleted.
        /// </summary>
	protected virtual void OnDeleteComplete()
	{
	}

        #endregion
    }
}
